import constants as const
import socket


class Node(object):
    def __init__(self, node_config, env):
        self.dst_dir = const.DST_DIR
        self.bash_script_path = None
        self.puppet_script_path = None
        self.selinux_script_path = None
        self.ospurge_script_path = None
        self.dhcp_reschedule_script_path = None
        self.dhcp_agent_scheduler_dir = None
        self.log = const.LOG_FILE
        self.hostname = node_config['hostname']
        self.fqdn = node_config.get('fqdn')
        if not self.fqdn:
            try:
                self.fqdn = socket.gethostbyaddr(self.hostname)[0]
            except Exception:
                self.fqdn = self.hostname
        self.uname = node_config.get('uname')
        self.role = node_config['role'].lower()
        self.skip = node_config['skip']
        self.deploy_mode = node_config.get('deploy_mode')
        self.os = node_config['os'].lower()
        self.os_version = str(node_config['os_version']).split(".")[0]
        self.user = node_config.get('user')
        self.passwd = node_config.get('passwd')
        self.uplink_interfaces = node_config.get('uplink_interfaces')
        self.uplink_mtu = node_config.get('uplink_mtu')
        if not self.uplink_mtu:
            self.uplink_mtu = 1500
        self.install_ivs = node_config.get('install_ivs')
        self.install_bsnstacklib = node_config.get('install_bsnstacklib')
        self.install_all = node_config.get('install_all')
        self.deploy_dhcp_agent = node_config.get('deploy_dhcp_agent')
        self.deploy_l3_agent = node_config.get('deploy_l3_agent')
        self.bridges = node_config.get('bridges')
        self.br_bond = node_config.get('br_bond')
        self.bond = node_config.get('bond')

        self.pxe_interface = node_config.get('pxe_interface')
        self.br_fw_admin = node_config.get('br_fw_admin')
        self.br_fw_admin_address = node_config.get('br_fw_admin_address')
        self.tagged_intfs = node_config.get('tagged_intfs')
        self.ex_gw = node_config.get('ex_gw')
        self.tag = node_config.get('tag')
        self.env_tag = env.tag
        self.cleanup = env.cleanup
        self.skip_ivs_version_check = env.skip_ivs_version_check
        self.rabbit_hosts = None

        # keystone client
        self.keystone_auth_url = None
        self.keystone_auth_user = None
        self.keystone_password = None
        self.keystone_auth_tenant = None

        # setup result
        self.time_diff = 0
        self.last_log = None

        # rhosp related config
        self.rhosp_automate_register = env.rhosp_automate_register
        self.rhosp_installer_management_interface = (
            env.rhosp_installer_management_interface)
        self.rhosp_installer_pxe_interface = env.rhosp_installer_pxe_interface
        self.rhosp_undercloud_dns = env.rhosp_undercloud_dns
        self.rhosp_register_username = env.rhosp_register_username
        self.rhosp_register_passwd = env.rhosp_register_passwd
        self.installer_pxe_interface_ip = env.installer_pxe_interface_ip

        self.neutron_id = env.neutron_id
        self.tenant_api_version = env.tenant_api_version
        self.openstack_release = env.openstack_release
        self.bsnstacklib_version_lower = env.bsnstacklib_version_lower
        self.bsnstacklib_version_upper = env.bsnstacklib_version_upper
        self.bcf_controllers = env.bcf_controllers
        self.bcf_controller_ips = env.bcf_controller_ips
        self.bcf_controller_user = env.bcf_controller_user
        self.bcf_controller_passwd = env.bcf_controller_passwd
        self.bcf_openstack_management_tenant = (
            env.bcf_openstack_management_tenant)
        self.bcf_master = env.bcf_master
        self.physnet = env.physnet
        self.lower_vlan = env.lower_vlan
        self.upper_vlan = env.upper_vlan
        self.setup_node_ip = env.setup_node_ip
        self.setup_node_dir = env.setup_node_dir
        self.selinux_mode = env.selinux_mode
        self.fuel_cluster_id = env.fuel_cluster_id
        self.rhosp = env.rhosp
        self.deploy_horizon_patch = env.deploy_horizon_patch
        self.horizon_patch_url = env.horizon_patch_url
        self.horizon_patch = env.horizon_patch
        self.horizon_patch_dir = env.horizon_patch_dir
        self.horizon_base_dir = env.horizon_base_dir
        self.ivs_pkg_map = env.ivs_pkg_map
        self.ivs_pkg = None
        self.ivs_debug_pkg = None
        self.ivs_version = None
        self.old_ivs_version = node_config.get('old_ivs_version')

        # in case of config env (packstack), bond and br_bond
        # may be empty
        if not self.br_bond:
            self.br_bond = const.T5_CENTOS_BOND_BRIDGE

        if not self.bond:
            self.bond = const.T5_CENTOS_BOND_NAME

        if self.os in const.RPM_OS_SET:
            self.ivs_pkg = self.ivs_pkg_map.get('rpm')
            self.ivs_debug_pkg = self.ivs_pkg_map.get('debug_rpm')
        elif self.os in const.DEB_OS_SET:
            self.ivs_pkg = self.ivs_pkg_map.get('deb')
            self.ivs_debug_pkg = self.ivs_pkg_map.get('debug_deb')
        self.error = node_config.get('error')

        # check os compatability
        if (((self.os == const.CENTOS) and
             (self.os_version not in const.CENTOS_VERSIONS))
            or ((self.os == const.UBUNTU) and
                (self.os_version not in const.UBUNTU_VERSIONS))
            or ((self.os == const.REDHAT) and
                (self.os_version not in const.REDHAT_VERSIONS))):
            self.skip = True
            self.error = (r'''%(os)s %(os_version)s is not supported''' %
                         {'os': self.os, 'os_version': self.os_version})

        # get ivs version
        if self.ivs_pkg:
            temp = []
            subs = self.ivs_pkg.split('-')
            for sub in subs:
                temp.extend(sub.split('_'))
            for i in range(len(temp)):
                if temp[i].lower() == 'ivs':
                    self.ivs_version = temp[i + 1]
                    break

        # check ivs compatability
        if self.deploy_mode == const.T6 and self.old_ivs_version:
            ivs_version_num = self.ivs_version.split('.')
            old_ivs_version_num = self.old_ivs_version.split('.')
            if not old_ivs_version_num[0].isdigit():
                return
            if old_ivs_version_num[0] == '0':
                return
            if str(self.ivs_version) in str(self.old_ivs_version):
                return
            if self.skip_ivs_version_check:
                return
            diff = int(ivs_version_num[0]) - int(old_ivs_version_num[0])
            if self.ivs_version < self.old_ivs_version:
                self.skip = True
                self.error = (r'''Existing ivs %(old_ivs_version)s is newer '''
                              '''than %(ivs_version)s''' %
                              {'old_ivs_version': self.old_ivs_version,
                               'ivs_version': self.ivs_version})
            elif diff > 1:
                self.skip = True
                self.error = (r'''Existing ivs %(old_ivs_version)s is '''
                              '''%(diff)d version behind %(ivs_version)s''' %
                              {'old_ivs_version': self.old_ivs_version,
                               'diff': diff,
                               'ivs_version': self.ivs_version})

    def set_bash_script_path(self, bash_script_path):
        self.bash_script_path = bash_script_path

    def set_puppet_script_path(self, puppet_script_path):
        self.puppet_script_path = puppet_script_path

    def set_selinux_script_path(self, selinux_script_path):
        self.selinux_script_path = selinux_script_path

    def set_ospurge_script_path(self, ospurge_script_path):
        self.ospurge_script_path = ospurge_script_path

    def set_dhcp_reschedule_script_path(self, dhcp_reschedule_script_path):
        self.dhcp_reschedule_script_path = dhcp_reschedule_script_path

    def set_dhcp_agent_scheduler_dir(self, dhcp_agent_scheduler_dir):
        self.dhcp_agent_scheduler_dir = dhcp_agent_scheduler_dir

    def set_time_diff(self, time_diff):
        self.time_diff = time_diff

    def set_last_log(self, last_log):
        self.last_log = last_log

    def set_rabbit_hosts(self, rabbit_hosts):
        self.rabbit_hosts = rabbit_hosts

    def set_keystone_auth_url(self, keystone_auth_url):
        self.keystone_auth_url = keystone_auth_url

    def set_keystone_auth_user(self, keystone_auth_user):
        self.keystone_auth_user = keystone_auth_user

    def set_keystone_password(self, keystone_password):
        self.keystone_password = keystone_password

    def set_keystone_auth_tenant(self, keystone_auth_tenant):
        self.keystone_auth_tenant = keystone_auth_tenant

    def get_network_vlan_ranges(self):
        return (r'''%(physnet)s:%(lower_vlan)s:%(upper_vlan)s''' %
               {'physnet': self.physnet,
                'lower_vlan': self.lower_vlan,
                'upper_vlan': self.upper_vlan})

    def get_bridge_mappings(self):
        return (r'''%(physnet)s:%(bond_bridge)s''' %
               {'physnet': self.physnet,
                'bond_bridge': self.br_bond})

    def get_uplink_intfs_for_ivs(self):
        uplink_intfs = []
        for intf in self.uplink_interfaces:
            uplink_intfs.append(' -u ')
            uplink_intfs.append(intf)
        return ''.join(uplink_intfs)

    def get_ivs_internal_ports(self):
        internal_ports = []
        if self.bridges:
            for br in self.bridges:
                if (not br.br_vlan) or (br.br_key == const.BR_KEY_PRIVATE):
                    continue
                prefixes = br.br_key.split('/')
                segment = prefixes[len(prefixes) - 1]
                port_prefix = const.IVS_INTERNAL_PORT_DIC.get(segment)
                if not port_prefix:
                    continue
                port = "%s%s" % (port_prefix, self.fuel_cluster_id)
                internal_ports.append(' --internal-port=')
                internal_ports.append(port)
        return ''.join(internal_ports)

    def get_ivs_internal_port_ips(self):
        port_ips = []
        if not self.bridges:
            return ' '.join(port_ips)
        for br in self.bridges:
            if ((not br.br_vlan) or (not br.br_ip) or
                    (br.br_key == const.BR_KEY_PRIVATE)):
                continue
            prefixes = br.br_key.split('/')
            segment = prefixes[len(prefixes) - 1]
            port_prefix = const.IVS_INTERNAL_PORT_DIC.get(segment)
            if not port_prefix:
                continue
            port = "%s%s" % (port_prefix, self.fuel_cluster_id)
            port_ips.append(r'''"%(port)s,%(ip)s"''' %
                           {'port': port,
                            'ip': br.br_ip})
        return ",".join(port_ips)

    def get_all_ovs_brs(self):
        ovs_brs = set()
        # ovs_brs.add(r'''"%(br)s"''' % {'br': const.BR_NAME_INT})
        if self.bridges:
            for br in self.bridges:
                ovs_brs.add(r'''"%(br)s"''' % {'br': br.br_name})
            ovs_brs.add(r'''"%(br)s"''' % {'br': self.br_bond})
        return ' '.join(ovs_brs)

    def get_all_interfaces(self):
        interfaces = []
        if self.pxe_interface:
            interfaces.append(self.pxe_interface)
        if self.uplink_interfaces:
            for intf in self.uplink_interfaces:
                interfaces.append(intf)
        if self.tagged_intfs:
            for intf in self.tagged_intfs:
                interfaces.append(intf)
        return ' '.join(interfaces)

    def get_all_uplinks(self):
        uplinks = []
        for intf in self.uplink_interfaces:
            uplinks.append(intf)
        return ' '.join(uplinks)

    def get_comma_separated_uplinks(self):
        uplinks = []
        for intf in self.uplink_interfaces:
            uplinks.append(intf)
        return ','.join(uplinks)

    def get_all_bonds(self):
        bonds = set()
        if self.bond and self.bridges:
            for br in self.bridges:
                if (br.br_vlan) and (':' not in str(br.br_vlan)):
                    bonds.add(r'''%(bond)s.%(vlan)s''' %
                              {'bond': self.bond,
                               'vlan': br.br_vlan})
            bonds.add(r'''%(bond)s''' % {'bond': self.bond})
        return ' '.join(bonds)

    def get_default_gw(self):
        if self.ex_gw:
            return self.ex_gw
        else:
            return self.installer_pxe_interface_ip

    def get_controllers_for_neutron(self):
        return ','.join(self.bcf_controllers)

    def get_neutron_id(self):
        if self.neutron_id and self.fuel_cluster_id:
            return "%s-%s" % (self.neutron_id, str(self.fuel_cluster_id))
        elif self.fuel_cluster_id:
            return "neutron-%s" % str(self.fuel_cluster_id)
        return self.neutron_id

    def get_bsnstacklib_version_lower(self):
        if self.openstack_release == const.OS_RELEASE_KILO:
            if self.tenant_api_version == const.TENANT_NAME_API_VERSION:
                return const.OS_RELEASE_TO_BSN_LIB_LOWER.get(
                    const.OS_RELEASE_KILO_V2)
            if self.tenant_api_version == const.TENANT_UUID_API_VERSION:
                return const.OS_RELEASE_TO_BSN_LIB_LOWER.get(
                    const.OS_RELEASE_KILO)
        return self.bsnstacklib_version_lower

    def get_bsnstacklib_version_upper(self):
        if self.openstack_release == const.OS_RELEASE_KILO:
            if self.tenant_api_version == const.TENANT_NAME_API_VERSION:
                return const.OS_RELEASE_TO_BSN_LIB_UPPER.get(
                    const.OS_RELEASE_KILO_V2)
            if self.tenant_api_version == const.TENANT_UUID_API_VERSION:
                return const.OS_RELEASE_TO_BSN_LIB_UPPER.get(
                    const.OS_RELEASE_KILO)
        return self.bsnstacklib_version_upper

    def __str__(self):
        return (
            r'''
            dst_dir: %(dst_dir)s,
            bash_script_path: %(bash_script_path)s,
            puppet_script_path: %(puppet_script_path)s,
            selinux_script_path: %(selinux_script_path)s,
            ospurge_script_path: %(ospurge_script_path)s,
            dhcp_reschedule_script_path: %(dhcp_reschedule_script_path)s,
            dhcp_agent_scheduler_dir: %(dhcp_agent_scheduler_dir)s,
            log: %(log)s,
            hostname: %(hostname)s,
            fqdn: %(fqdn)s,
            uname: %(uname)s,
            role: %(role)s,
            skip: %(skip)s,
            deploy_mode: %(deploy_mode)s,
            os: %(os)s,
            os_version: %(os_version)s,
            user: %(user)s,
            passwd: %(passwd)s,
            uplink_interfaces: %(uplink_interfaces)s,
            uplink_mtu: %(uplink_mtu)s,
            install_ivs: %(install_ivs)s,
            install_bsnstacklib: %(install_bsnstacklib)s,
            install_all: %(install_all)s,
            deploy_dhcp_agent: %(deploy_dhcp_agent)s,
            deploy_l3_agent: %(deploy_l3_agent)s,
            bridges: %(bridges)s,
            br_bond: %(br_bond)s,
            bond: %(bond)s,
            pxe_interface: %(pxe_interface)s,
            br_fw_admin: %(br_fw_admin)s,
            br_fw_admin_address: %(br_fw_admin_address)s,
            tagged_intfs: %(tagged_intfs)s,
            ex_gw: %(ex_gw)s,
            tag: %(tag)s,
            env_tag: %(env_tag)s,
            cleanup: %(cleanup)s,
            rabbit_hosts: %(rabbit_hosts)s,
            keystone_auth_url: %(keystone_auth_url)s,
            keystone_auth_user: %(keystone_auth_user)s,
            keystone_password: %(keystone_password)s,
            keystone_auth_tenant: %(keystone_auth_tenant)s,
            time_diff: %(time_diff)s,
            last_log: %(last_log)s,
            rhosp_automate_register: %(rhosp_automate_register)s,
            rhosp_installer_management_interface:
            %(rhosp_installer_management_interface)s,
            rhosp_installer_pxe_interface:
            %(rhosp_installer_pxe_interface)s,
            rhosp_undercloud_dns: %(rhosp_undercloud_dns)s,
            rhosp_register_username: %(rhosp_register_username)s,
            rhosp_register_passwd: %(rhosp_register_passwd)s,
            installer_pxe_interface_ip: %(installer_pxe_interface_ip)s,
            neutron_id: %(neutron_id)s,
            tenant_api_version: %(tenant_api_version)s,
            openstack_release: %(openstack_release)s,
            bsnstacklib_version_lower: %(bsnstacklib_version_lower)s,
            bsnstacklib_version_upper: %(bsnstacklib_version_upper)s,
            bcf_controllers: %(bcf_controllers)s,
            bcf_controller_ips: %(bcf_controller_ips)s,
            bcf_controller_user: %(bcf_controller_user)s,
            bcf_controller_passwd: %(bcf_controller_passwd)s,
            bcf_openstack_management_tenant:
            %(bcf_openstack_management_tenant)s,
            bcf_master: %(bcf_master)s,
            physnet: %(physnet)s,
            lower_vlan: %(lower_vlan)s,
            upper_vlan: %(upper_vlan)s,
            setup_node_ip: %(setup_node_ip)s,
            setup_node_dir: %(setup_node_dir)s,
            selinux_mode: %(selinux_mode)s,
            fuel_cluster_id: %(fuel_cluster_id)s,
            rhosp: %(rhosp)s,
            deploy_horizon_patch: %(deploy_horizon_patch)s,
            horizon_patch_url: %(horizon_patch_url)s,
            horizon_patch: %(horizon_patch)s,
            horizon_patch_dir: %(horizon_patch_dir)s,
            horizon_base_dir: %(horizon_base_dir)s,
            ivs_pkg: %(ivs_pkg)s,
            ivs_debug_pkg: %(ivs_debug_pkg)s,
            ivs_version: %(ivs_version)s,
            old_ivs_version: %(old_ivs_version)s,
            error: %(error)s,
            ''' %
            {'dst_dir': self.dst_dir,
            'bash_script_path': self.bash_script_path,
            'puppet_script_path': self.puppet_script_path,
            'selinux_script_path': self.selinux_script_path,
            'ospurge_script_path': self.ospurge_script_path,
            'dhcp_reschedule_script_path': self.dhcp_reschedule_script_path,
            'dhcp_agent_scheduler_dir': self.dhcp_agent_scheduler_dir,
            'log': self.log,
            'hostname': self.hostname,
            'fqdn': self.fqdn,
            'uname': self.uname,
            'role': self.role,
            'skip': self.skip,
            'deploy_mode': self.deploy_mode,
            'os': self.os,
            'os_version': self.os_version,
            'user': self.user,
            'passwd': self.passwd,
            'uplink_interfaces': self.uplink_interfaces,
            'uplink_mtu': self.uplink_mtu,
            'install_ivs': self.install_ivs,
            'install_bsnstacklib': self.install_bsnstacklib,
            'install_all': self.install_all,
            'deploy_dhcp_agent': self.deploy_dhcp_agent,
            'deploy_l3_agent': self.deploy_l3_agent,
            'bridges': str(self.bridges),
            'br_bond': self.br_bond,
            'bond': self.bond,
            'pxe_interface': self.pxe_interface,
            'br_fw_admin': self.br_fw_admin,
            'br_fw_admin_address': self.br_fw_admin_address,
            'tagged_intfs': self.tagged_intfs,
            'ex_gw': self.ex_gw,
            'tag': self.tag,
            'env_tag': self.env_tag,
            'cleanup': self.cleanup,
            'rabbit_hosts': self.rabbit_hosts,
            'keystone_auth_url': self.keystone_auth_url,
            'keystone_auth_user': self.keystone_auth_user,
            'keystone_password': self.keystone_password,
            'keystone_auth_tenant': self.keystone_auth_tenant,
            'time_diff': self.time_diff,
            'last_log': self.last_log,
            'rhosp_automate_register': self.rhosp_automate_register,
            'rhosp_installer_management_interface':
                self.rhosp_installer_management_interface,
            'rhosp_installer_pxe_interface':
                self.rhosp_installer_pxe_interface,
            'rhosp_undercloud_dns': self.rhosp_undercloud_dns,
            'rhosp_register_username': self.rhosp_register_username,
            'rhosp_register_passwd': self.rhosp_register_passwd,
            'installer_pxe_interface_ip': self.installer_pxe_interface_ip,
            'neutron_id': self.neutron_id,
            'tenant_api_version': self.tenant_api_version,
            'openstack_release': self.openstack_release,
            'bsnstacklib_version_lower': self.get_bsnstacklib_version_lower(),
            'bsnstacklib_version_upper': self.get_bsnstacklib_version_upper(),
            'bcf_controllers': self.bcf_controllers,
            'bcf_controller_ips': self.bcf_controller_ips,
            'bcf_controller_user': self.bcf_controller_user,
            'bcf_controller_passwd': self.bcf_controller_passwd,
            'bcf_openstack_management_tenant':
                self.bcf_openstack_management_tenant,
            'bcf_master': self.bcf_master,
            'physnet': self.physnet,
            'lower_vlan': self.lower_vlan,
            'upper_vlan': self.upper_vlan,
            'setup_node_ip': self.setup_node_ip,
            'setup_node_dir': self.setup_node_dir,
            'selinux_mode': self.selinux_mode,
            'fuel_cluster_id': self.fuel_cluster_id,
            'rhosp': self.rhosp,
            'deploy_horizon_patch': self.deploy_horizon_patch,
            'horizon_patch_url': self.horizon_patch_url,
            'horizon_patch': self.horizon_patch,
            'horizon_patch_dir': self.horizon_patch_dir,
            'horizon_base_dir': self.horizon_base_dir,
            'ivs_pkg': self.ivs_pkg,
            'ivs_debug_pkg': self.ivs_debug_pkg,
            'ivs_version': self.ivs_version,
            'old_ivs_version': self.old_ivs_version,
            'error': self.error})

    def __repr__(self):
        return self.__str__()



import constants as const
import httplib
import json
from util import safe_print


class RestLib(object):
    @staticmethod
    def request(url, prefix="/api/v1/data/controller/", method='GET',
                data='', hashPath=None, host="127.0.0.1:8443", cookie=None):
        headers = {'Content-type': 'application/json'}

        if cookie:
            headers['Cookie'] = 'session_cookie=%s' % cookie

        if hashPath:
            headers[const.HASH_HEADER] = hashPath

        connection = httplib.HTTPSConnection(host)

        try:
            connection.request(method, prefix + url, data, headers)
            response = connection.getresponse()
            ret = (response.status, response.reason, response.read(),
                   response.getheader(const.HASH_HEADER))
            with open(const.LOG_FILE, "a") as log_file:
                log_file.write('Controller REQUEST: %s %s:body=%r\n' %
                              (method, host + prefix + url, data))
                log_file.write('Controller RESPONSE: status=%d reason=%r,'
                               'data=%r, hash=%r\n' % ret)
            return ret
        except Exception as e:
            raise Exception("Controller REQUEST exception: %s" % e)

    @staticmethod
    def get(cookie, url, server, port, hashPath=None):
        host = "%s:%d" % (server, port)
        return RestLib.request(url, hashPath=hashPath, host=host,
                               cookie=cookie)

    @staticmethod
    def post(cookie, url, server, port, data, hashPath=None):
        host = "%s:%d" % (server, port)
        return RestLib.request(url, method='POST', hashPath=hashPath,
                               host=host, data=data, cookie=cookie)

    @staticmethod
    def patch(cookie, url, server, port, data, hashPath=None):
        host = "%s:%d" % (server, port)
        return RestLib.request(url, method='PATCH', hashPath=hashPath,
                               host=host, data=data, cookie=cookie)

    @staticmethod
    def put(cookie, url, server, port, data, hashPath=None):
        host = "%s:%d" % (server, port)
        return RestLib.request(url, method='PUT', hashPath=hashPath,
                               host=host, data=data, cookie=cookie)

    @staticmethod
    def delete(cookie, url, server, port, hashPath=None):
        host = "%s:%d" % (server, port)
        return RestLib.request(url, method='DELETE', hashPath=hashPath,
                               host=host, cookie=cookie)

    @staticmethod
    def auth_bcf(server, username, password, port=const.BCF_CONTROLLER_PORT):
        login = {"user": username, "password": password}
        host = "%s:%d" % (server, port)
        ret = RestLib.request("/api/v1/auth/login", prefix='',
                              method='POST', data=json.dumps(login),
                              host=host)
        session = json.loads(ret[2])
        if ret[0] != 200:
            raise Exception(ret)
        if ("session_cookie" not in session):
            raise Exception("Failed to authenticate: session cookie not set")
        return session["session_cookie"]

    @staticmethod
    def logout_bcf(cookie, server, port=const.BCF_CONTROLLER_PORT):
        url = "core/aaa/session[auth-token=\"%s\"]" % cookie
        ret = RestLib.delete(cookie, url, server, port)
        return ret

    @staticmethod
    def get_active_bcf_controller(servers, username, password,
                                  port=const.BCF_CONTROLLER_PORT):
        for server in servers:
            try:
                cookie = RestLib.auth_bcf(server, username, password, port)
                url = 'core/controller/role'
                res = RestLib.get(cookie, url, server, port)[2]
                if 'active' in res:
                    return server, cookie
            except Exception:
                continue
        return None, None

    @staticmethod
    def get_os_mgmt_segments(server, cookie, tenant,
                             port=const.BCF_CONTROLLER_PORT):
        url = (r'''applications/bcf/info/endpoint-manager/segment'''
               '''[tenant="%(tenant)s"]''' %
               {'tenant': tenant})
        ret = RestLib.get(cookie, url, server, port)
        if ret[0] != 200:
            raise Exception(ret)
        res = json.loads(ret[2])
        segments = []
        for segment in res:
            # 'management' or 'Management' segment does not matter
            segments.append(segment['name'].lower())
        return segments

    @staticmethod
    def program_segment_and_membership_rule(server, cookie, rule, tenant,
                                            port=const.BCF_CONTROLLER_PORT):
        if rule.segment not in const.IVS_INTERNAL_PORT_DIC:
            return

        existing_segments = RestLib.get_os_mgmt_segments(
            server, cookie, tenant, port)
        if rule.segment not in existing_segments:
            with open(const.LOG_FILE, "a") as log_file:
                msg = (r'''Warning: BCF controller does not have tenant '''
                       '''%(tenant)s segment %(segment)s\n''' %
                       {'tenant': tenant, 'segment': rule.segment})
                safe_print(msg)
                log_file.write(msg)
            return

        segment_url = (r'''applications/bcf/tenant[name="%(tenant)s"]/'''
                       '''segment[name="%(segment)s"]''' %
                       {'tenant': tenant, 'segment': rule.segment})
        segment_data = {"name": rule.segment}
        safe_print("Configuring BCF Segment: Tenant %s, Segment %s\n" %
                   (tenant, rule.segment))
        ret = RestLib.post(cookie, segment_url, server, port,
                           json.dumps(segment_data))
        if ret[0] != 204:
            raise Exception(ret)

        if rule.br_vlan:
            vlan = int(rule.br_vlan)
        else:
            vlan = -1

        intf_rule_url = (r'''applications/bcf/tenant[name="%(tenant)s"]/'''
                         '''segment[name="%(segment)s"]/'''
                         '''switch-port-membership-rule[interface="'''
                         '''%(interface)s"][switch="%(switch)s"]'''
                         '''[vlan=%(vlan)d]''' %
                         {'tenant': tenant,
                          'segment': rule.segment,
                          'interface': const.ANY,
                          'switch': const.ANY,
                          'vlan': vlan})
        rule_data = {"interface": const.ANY, "switch": const.ANY, "vlan": vlan}
        safe_print("Configuring BCF Segment rule: Tenant %s, Segment "
                   "%s Rule: member switch any interface any vlan %d\n"
                   % (tenant, rule.segment, vlan))
        ret = RestLib.post(cookie, intf_rule_url, server, port,
                           json.dumps(rule_data))
        if ret[0] != 204:
            raise Exception(ret)

        pg_rule_url = (r'''applications/bcf/tenant[name="%(tenant)s"]/'''
                       '''segment[name="%(segment)s"]/'''
                       '''port-group-membership-rule[port-group="%(pg)s"]'''
                       '''[vlan=%(vlan)d]''' %
                       {'tenant': tenant,
                        'segment': rule.segment,
                        'pg': const.ANY,
                        'vlan': vlan})
        rule_data = {"port-group": const.ANY, "vlan": vlan}
        safe_print("Configuring BCF Segment rule: Tenant %s, "
                   "Segment %s Rule: member port-group any vlan %d\n"
                   % (tenant, rule.segment, vlan))
        ret = RestLib.post(cookie, pg_rule_url, server, port,
                           json.dumps(rule_data))
        if ret[0] != 204:
            raise Exception(ret)

        specific_rule_url = (r'''applications/bcf/tenant[name="%(tenant)s"]/'''
                             '''segment[name="%(segment)s"]/'''
                             '''switch-port-membership-rule'''
                             '''[interface="%(interface)s"]'''
                             '''[switch="%(switch)s"][vlan=%(vlan)d]''' %
                             {'tenant': tenant,
                              'segment': rule.segment,
                              'interface': rule.internal_port,
                              'switch': const.ANY,
                              'vlan': -1})
        rule_data = {"interface": rule.internal_port,
                     "switch": const.ANY, "vlan": -1}
        safe_print("Configuring BCF Segment rule: Tenant %s, Segment %s Rule: "
                   "member switch any interface %s vlan untagged\n"
                   % (tenant, rule.segment, rule.internal_port))
        ret = RestLib.post(cookie, specific_rule_url, server, port,
                           json.dumps(rule_data))
        if ret[0] != 204:
            raise Exception(ret)



import constants as const
from helper import Helper
import os
import re
from rest import RestLib


class Environment(object):
    def __init__(self, config, mode, fuel_cluster_id, rhosp, tag, cleanup, skip_ivs_version_check):
        # fuel cluster id
        self.fuel_cluster_id = fuel_cluster_id

        # rhosp as installer
        self.rhosp = rhosp

        # tag, only deploy nodes with this tag
        self.tag = tag

        # clean up flag
        self.cleanup = cleanup

        self.skip_ivs_version_check = skip_ivs_version_check

        # neutron_id for ml2 plugin restproxy
        self.neutron_id = config.get('neutron_id')

        # tenant api version
        self.tenant_api_version = config.get('tenant_api_version')
        if not self.tenant_api_version:
            self.tenant_api_version = const.TENANT_NAME_API_VERSION

        # installer pxe interface ip
        self.installer_pxe_interface_ip = config.get(
            'installer_pxe_interface_ip')

        # install to specified nodes
        self.deploy_to_specified_nodes_only = config.get(
            'deploy_to_specified_nodes_only')

        # flags for upgrade
        self.install_ivs = config.get('default_install_ivs')
        self.install_bsnstacklib = config.get('default_install_bsnstacklib')
        self.install_all = True

        # flags for dhcp and metadata agent
        self.deploy_dhcp_agent = config.get('default_deploy_dhcp_agent')

        # flags for l3 agent
        self.deploy_l3_agent = config.get('default_deploy_l3_agent')

        # setup node ip and directory
        self.setup_node_ip = Helper.get_setup_node_ip()
        example_yamls = ["/usr/local/etc/bosi/config.yaml",
                         "/usr/etc/bosi/config.yaml"]
        for example_yaml in example_yamls:
            if os.path.isfile(example_yaml):
                self.setup_node_dir = os.path.dirname(example_yaml)
                break

        # t5 or t6 mode
        self.deploy_mode = const.MODE_DICT.get(mode)
        if not self.deploy_mode:
            self.deploy_mode = const.T5

        # selinux configuration
        self.selinux_mode = None
        if os.path.isfile(const.SELINUX_CONFIG_PATH):
            with open(const.SELINUX_CONFIG_PATH, "r") as selinux_config_file:
                selinux_mode_match = re.compile(const.SELINUX_MODE_EXPRESSION,
                                                re.IGNORECASE)
                lines = selinux_config_file.readlines()
                for line in lines:
                    match = selinux_mode_match.match(line)
                    if match:
                        self.selinux_mode = match.group(1)
                        break

        # neutron vlan ranges
        self.network_vlan_ranges = config.get('network_vlan_ranges')
        network_vlan_range_pattern = re.compile(
            const.NETWORK_VLAN_RANGE_EXPRESSION, re.IGNORECASE)
        match = network_vlan_range_pattern.match(self.network_vlan_ranges)
        if not match:
            Helper.safe_print("network_vlan_ranges' format is not correct.\n")
            exit(1)
        self.physnet = match.group(1)
        self.lower_vlan = match.group(2)
        self.upper_vlan = match.group(3)

        # bcf controller information
        self.bcf_controllers = config['bcf_controllers']
        self.bcf_controller_ips = []
        for controller in self.bcf_controllers:
            ip = controller.split(':')[0]
            self.bcf_controller_ips.append(ip)
        self.bcf_controller_user = config['bcf_controller_user']
        self.bcf_controller_passwd = config['bcf_controller_passwd']
        self.bcf_openstack_management_tenant = config.get(
            'bcf_openstack_management_tenant')

        # ivs pkg and debug pkg
        self.ivs_pkg_map = {}
        self.ivs_url_map = {}
        if config['ivs_packages']:
            for ivs_url in config['ivs_packages']:
                ivs_pkg = os.path.basename(ivs_url)
                if '.rpm' in ivs_pkg and 'debuginfo' not in ivs_pkg:
                    self.ivs_url_map['rpm'] = ivs_url
                    self.ivs_pkg_map['rpm'] = ivs_pkg
                elif '.rpm' in ivs_pkg and 'debuginfo' in ivs_pkg:
                    self.ivs_url_map['debug_rpm'] = ivs_url
                    self.ivs_pkg_map['debug_rpm'] = ivs_pkg
                elif '.deb' in ivs_pkg and 'dbg' not in ivs_pkg:
                    self.ivs_url_map['deb'] = ivs_url
                    self.ivs_pkg_map['deb'] = ivs_pkg
                elif '.deb' in ivs_pkg and 'dbg' in ivs_pkg:
                    self.ivs_url_map['debug_deb'] = ivs_url
                    self.ivs_pkg_map['debug_deb'] = ivs_pkg
                elif '.tar.gz' in ivs_pkg:
                    self.ivs_url_map['tar'] = ivs_url
                    self.ivs_pkg_map['tar'] = ivs_pkg

        # information will be passed on to nodes
        self.skip = False
        if 'default_skip' in config:
            self.skip = config['default_skip']
        self.os = config.get('default_os')
        self.os_version = config.get('default_os_version')
        self.role = config.get('default_role')
        self.user = config.get('default_user')
        self.passwd = config.get('default_passwd')
        self.uplink_interfaces = config.get('default_uplink_interfaces')
        self.uplink_mtu = config.get('default_uplink_mtu')

        # openstack bsnstacklib version and horizon patch
        self.openstack_release = str(config['openstack_release']).lower()
        self.bsnstacklib_version_lower = (
            const.OS_RELEASE_TO_BSN_LIB_LOWER[self.openstack_release])
        self.bsnstacklib_version_upper = (
            const.OS_RELEASE_TO_BSN_LIB_UPPER[self.openstack_release])
        self.deploy_horizon_patch = const.DEPLOY_HORIZON_PATCH
        self.horizon_patch_url = (
            const.HORIZON_PATCH_URL[self.openstack_release])
        self.horizon_patch = os.path.basename(self.horizon_patch_url)
        self.horizon_patch_dir = (
            const.HORIZON_PATCH_DIR[self.openstack_release])
        self.horizon_base_dir = const.HORIZON_BASE_DIR

        # master bcf controller and cookie
        self.bcf_master = None
        self.bcf_cookie = None
        if fuel_cluster_id:
            self.bcf_master, self.bcf_cookie = (
                RestLib.get_active_bcf_controller(self.bcf_controller_ips,
                                                  self.bcf_controller_user,
                                                  self.bcf_controller_passwd))
            if (not self.bcf_master) or (not self.bcf_cookie):
                raise Exception("Failed to connect to master BCF controller, "
                                "quit setup.")

        # RHOSP 7 related config
        self.rhosp_automate_register = False
        if 'rhosp_automate_register' in config:
            self.rhosp_automate_register = config['rhosp_automate_register']
        self.rhosp_installer_management_interface = config.get(
            'rhosp_installer_management_interface')
        self.rhosp_installer_pxe_interface = config.get(
            'rhosp_installer_pxe_interface')
        self.rhosp_undercloud_dns = config.get('rhosp_undercloud_dns')
        self.rhosp_register_username = config.get('rhosp_register_username')
        self.rhosp_register_passwd = config.get('rhosp_register_passwd')

    def set_physnet(self, physnet):
        self.physnet = physnet

    def set_lower_vlan(self, lower_vlan):
        self.lower_vlan = lower_vlan

    def set_upper_vlan(self, upper_vlan):
        self.upper_vlan = upper_vlan

    def set_ivs_pkg_map(self, ivs_pkg):
        if '.rpm' in ivs_pkg and 'debuginfo' not in ivs_pkg:
            self.ivs_pkg_map['rpm'] = ivs_pkg
        elif '.rpm' in ivs_pkg and 'debuginfo' in ivs_pkg:
            self.ivs_pkg_map['debug_rpm'] = ivs_pkg
        elif '.deb' in ivs_pkg and 'dbg' not in ivs_pkg:
            self.ivs_pkg_map['deb'] = ivs_pkg
        elif '.deb' in ivs_pkg and 'dbg' in ivs_pkg:
            self.ivs_pkg_map['debug_deb'] = ivs_pkg



# uname -n cutoff length for port group name
UNAME_CUTOFF = 249

# max number of threads, each thread sets up one node
MAX_WORKERS = 10

# root access to all the nodes is required
DEFAULT_USER = 'root'

# key words to specify node role in yaml config
ROLE_NEUTRON_SERVER = 'controller'
ROLE_COMPUTE = 'compute'
ROLE_CEPH = 'ceph-osd'

# deployment t6/t5
T6 = 't6'
T5 = 't5'

MODE_DICT = {'pfabric': T5,
             'pvfabric': T6}

OS_RELEASE_JUNO = 'juno'
OS_RELEASE_KILO = 'kilo'
OS_RELEASE_KILO_V2 = 'kilo_v2'
OS_RELEASE_LIBERTY = 'liberty'

# openstack release to bsnstacklib version
OS_RELEASE_TO_BSN_LIB_LOWER = {OS_RELEASE_JUNO: '2015.0',
                               OS_RELEASE_KILO_V2: '2015.1',
                               OS_RELEASE_KILO: '2015.2',
                               OS_RELEASE_LIBERTY: '2016.0'}
OS_RELEASE_TO_BSN_LIB_UPPER = {OS_RELEASE_JUNO: '2015.1',
                               OS_RELEASE_KILO_V2: '2015.2',
                               OS_RELEASE_KILO: '2015.3',
                               OS_RELEASE_LIBERTY: '2016.1'}

# Since kilo and BCF 3.5, we use tenant name
# instead of tenant uuid to configure tenants,
# The default version is 2. However, in case
# of upgrade, where tenant configuraion was
# using uuid, user needs to use version 1
# to make upgrade happen.
TENANT_UUID_API_VERSION = 1
TENANT_NAME_API_VERSION = 2

IVS_TAR_PKG_DIRS = ["pkg/centos7-x86_64", "pkg/trusty-amd64"]

# horizon patch
DEPLOY_HORIZON_PATCH = True
HORIZON_PATCH_URL = {
    OS_RELEASE_JUNO: 'https://github.com/bigswitch/horizon/archive/'
                     'juno-bcf-3.0-beta1.tar.gz',
    OS_RELEASE_KILO: 'https://github.com/bigswitch/horizon/archive/'
                     'stable/kilo2.tar.gz',
}
HORIZON_PATCH_DIR = {
    OS_RELEASE_JUNO: 'horizon-juno-bcf-3.0-beta1',
    OS_RELEASE_KILO: 'horizon-stable-kilo2',
}
HORIZON_BASE_DIR = '/usr/share/openstack-dashboard'

# constant file, directory names for each node
PRE_REQUEST_BASH = 'pre_request.sh'
DST_DIR = '/tmp'
GENERATED_SCRIPT_DIR = 'generated_script'
BASH_TEMPLATE_DIR = 'bash_template'
PYTHON_TEMPLATE_DIR = 'python_template'
PUPPET_TEMPLATE_DIR = 'puppet_template'
SELINUX_TEMPLATE_DIR = 'selinux_template'
OSPURGE_TEMPLATE_DIR = 'ospurge_template'
LOG_FILE = "/var/log/bcf_setup.log"

# constants for ivs config
INBAND_VLAN = 4092
IVS_DAEMON_ARGS = (r'''DAEMON_ARGS=\"--hitless --inband-vlan %(inband_vlan)d'''
                   '''%(uplink_interfaces)s%(internal_ports)s\\"''')

# constants of supported OSes and versions
CENTOS = 'centos'
CENTOS_VERSIONS = ['7']
UBUNTU = 'ubuntu'
UBUNTU_VERSIONS = ['14']
REDHAT = 'redhat'
REDHAT_VERSIONS = ['7']

# OSes that uses rpm or deb packages
RPM_OS_SET = [CENTOS, REDHAT]
DEB_OS_SET = [UBUNTU]

# regular expressions
EXISTING_NETWORK_VLAN_RANGE_EXPRESSION = (
    '^\s*network_vlan_ranges\s*=\s*(\S*)\s*:\s*(\S*)\s*:\s*(\S*)\s*$')
NETWORK_VLAN_RANGE_EXPRESSION = '^\s*(\S*)\s*:\s*(\S*)\s*:\s*(\S*)\s*$'
VLAN_RANGE_CONFIG_PATH = '/etc/neutron/plugins/ml2/ml2_conf.ini'
SELINUX_MODE_EXPRESSION = '^\s*SELINUX\s*=\s*(\S*)\s*$'
SELINUX_CONFIG_PATH = '/etc/selinux/config'


# openrc
FUEL_OPENRC = '/root/openrc'
PACKSTACK_OPENRC = '/root/keystonerc_admin'
MANUAL_OPENRC = '/root/admin-openrc.sh'
RHOSP_UNDERCLOUD_OPENRC = '/home/stack/stackrc'
RHOSP_OVERCLOUD_OPENRC = '/home/stack/overcloudrc'

# fuel constants
NONE_IP = 'none'
BR_KEY_PRIVATE = 'neutron/private'
BR_KEY_FW_ADMIN = 'fw-admin'
BR_NAME_INT = 'br-int'

# ivs internal port prefix mapping
IVS_INTERNAL_PORT_DIC = {
    'management': 'm',
    'ex': 'e',
    'storage': 's'}

HASH_HEADER = 'BCF-SETUP'
BCF_CONTROLLER_PORT = 8443
ANY = 'any'

# T5 for Centos requires extra params when using packstack
T5_CENTOS_BOND_BRIDGE = 'br-bond0'
T5_CENTOS_BOND_NAME = 'bond0'



import constants as const


class MembershipRule(object):
    def __init__(self, br_key, br_vlan, tenant, cluster_id):
        prefixes = br_key.split('/')
        segment = prefixes[len(prefixes) - 1]
        self.segment = segment
        self.br_vlan = br_vlan
        self.tenant = tenant
        self.cluster_id = cluster_id
        internal_port_prefix = const.IVS_INTERNAL_PORT_DIC.get(segment)
        self.internal_port = "%s%s" % (internal_port_prefix, cluster_id)

    def __str__(self):
        return (r'''{segment: %(segment)s, br_vlan: %(br_vlan)s, '''
                '''cluster_id: %(cluster_id)s, internal_port: '''
                '''%(internal_port)s}''' %
               {'segment': self.segment, 'br_vlan': self.br_vlan,
                'tenant': self.tenant, 'cluster_id': self.cluster_id,
                'internal_port': self.internal_port})

    def __repr__(self):
        return self.__str__()



import subprocess32 as subprocess
import sys

from threading import Lock

__print_lock = Lock()


def safe_print(message):
    """
    Grab the lock and print to stdout.
    The lock is to serialize messages from
    different thread. 'stty sane' is to
    clean up any hiden space.
    """
    with __print_lock:
        subprocess.call('stty sane', shell=True)
        sys.stdout.write(message)
        sys.stdout.flush()
        subprocess.call('stty sane', shell=True)



class Bridge(object):
    def __init__(self, br_key, br_name, br_ip, br_vlan):
        self.br_key = br_key
        self.br_name = br_name
        self.br_ip = br_ip
        self.br_vlan = br_vlan

    def __str__(self):
        return (r'''{br_key : %(br_key)s, br_name : %(br_name)s, br_ip :
                %(br_ip)s, br_vlan : %(br_vlan)s}''' %
                {'br_key': self.br_key, 'br_name': self.br_name,
                 'br_ip': self.br_ip, 'br_vlan': self.br_vlan})

    def __repr__(self):
        return self.__str__()
